@@ -75,6 +75,7 @@ class SubprocessTests(object):
         self.sock.close()
         # This give us one FD back ...
         self.queue.close()
+        self.proc.join()
 
     def assertline(self, line, status, reason, version):
         v, s, r = (x.strip() for x in line.split(None, 2))
@@ -125,6 +126,7 @@ class SleepyThreadTests(TcpTests, unittest.TestCase):
         for proc in procs:
             if proc.returncode is not None: # pragma: no cover
                 proc.terminate()
+            proc.wait()
         # the notsleepy response should always be first returned (it sleeps
         # for 2 seconds, then returns; the notsleepy response should be
         # processed in the meantime)
@@ -155,7 +157,7 @@ class EchoTests(object):
         self.assertTrue(headers.get('date'))
 
     def test_bad_host_header(self):
-        # http://corte.si/posts/code/pathod/pythonservers/index.html
+        # https://corte.si/posts/code/pathod/pythonservers/index.html
         to_send = ("GET / HTTP/1.0\n"
                    " Host: 0\n\n")
         to_send = tobytes(to_send)
@@ -193,16 +195,20 @@ class EchoTests(object):
         self.assertEqual(response_body, b'')
 
     def test_multiple_requests_with_body(self):
+        orig_sock = self.sock
         for x in range(3):
             self.sock = self.create_socket()
             self.test_send_with_body()
             self.sock.close()
+        self.sock = orig_sock
 
     def test_multiple_requests_without_body(self):
+        orig_sock = self.sock
         for x in range(3):
             self.sock = self.create_socket()
             self.test_send_empty_body()
             self.sock.close()
+        self.sock = orig_sock
 
     def test_without_crlf(self):
         data = "Echo\nthis\r\nplease"
@@ -252,6 +258,8 @@ class EchoTests(object):
             responses.append(response)
         for response in responses:
             response.read()
+        for h in conns:
+            h.close()
 
     def test_chunking_request_without_content(self):
         header = tobytes(
@@ -1223,6 +1231,24 @@ class FileWrapperTests(object):
             self.assertTrue(b'\377\330\377' in response_body)
             # connection has not been closed
 
+    def test_notfilelike_iobase_http11(self):
+        to_send = "GET /notfilelike_iobase HTTP/1.1\n\n"
+        to_send = tobytes(to_send)
+
+        self.connect()
+
+        for t in range(0, 2):
+            self.sock.send(to_send)
+            fp = self.sock.makefile('rb', 0)
+            line, headers, response_body = read_http(fp)
+            self.assertline(line, '200', 'OK', 'HTTP/1.1')
+            cl = int(headers['content-length'])
+            self.assertEqual(cl, len(response_body))
+            ct = headers['content-type']
+            self.assertEqual(ct, 'image/jpeg')
+            self.assertTrue(b'\377\330\377' in response_body)
+            # connection has not been closed
+
     def test_notfilelike_nocl_http11(self):
         to_send = "GET /notfilelike_nocl HTTP/1.1\n\n"
         to_send = tobytes(to_send)
